## 付録 デバッグ用平面を非表示にする
　第4章まででゲームとしては遊べるようになりましたが、端末でAR表示をする際のデバッグ用の平面が残ってしまっており、少し格好悪いですよね。

　そこで、平面を非表示にする機能を追加してみましょう。操作としては下スワイプ入力を新たに追加し、下スワイプを入力するたびに平面の表示非表示が交互に切り替わるように実装していきます。

　GameManagerに以下のように追記します。

```
using UnityEngine;
using System.Collections.Generic;
using UnityEngine.XR.ARFoundation;
using UnityEngine.XR.ARSubsystems;

public class GameManager : MonoBehaviour
{
    [SerializeField]
    GameObject goal;

    [SerializeField]
    GameObject ball;

    float maxRayDistance = 30.0f;

    ARRaycastManager raycastManager;
+    ARPlaneManager planeManager; // ①
    List<ARRaycastHit> hitList = new List<ARRaycastHit>();

    private Vector3 touchStartPos;
    private Vector3 touchEndPos;


    void Start()
    {
        raycastManager = GetComponent<ARRaycastManager>();
+        planeManager = GetComponent<ARPlaneManager>();
    }
    void Update()
    {
#if UNITY_EDITOR
        // UnityEditor上でARの挙動をシミュレート
        if (Input.GetMouseButtonDown(0))
        {
            touchStartPos = Input.mousePosition;
        }
        if (Input.GetMouseButtonUp(0))
        {
            touchEndPos = Input.mousePosition;
            var flickDistance = +touchEndPos.y - touchStartPos.y;
            if (flickDistance > 0)
            {
                CreateBall(flickDistance);
            }
            else
            {
                Ray ray = Camera.main.ScreenPointToRay(
                    Input.mousePosition);
                RaycastHit hit;
                if (Physics.Raycast(ray, out hit,
                    maxRayDistance))
                {
                    SetGoal(hit.point);
                }
            }
        }
#else
          // iOS端末上での動作
        if (Input.touchCount > 0)
        {
            var touch = Input.GetTouch(0);
            if (touch.phase == TouchPhase.Began)
            {
                touchStartPos = touch.position;
            }
            if (touch.phase == TouchPhase.Ended)
            {
                touchEndPos = touch.position;
                var flickDistance =
                                   touchEndPos.y - touchStartPos.y;
                if (flickDistance > 0)
                {
                    CreateBall(flickDistance);
                }
+              else if (flickDistance == 0)
                {

                    if (raycastManager.Raycast(
                        touch.position, hitList,
                        TrackableType.Planes))
                    {
                        SetGoal(hitList[0].pose.position);
                    }

                }
                else
                {
+                  // 下フリック
+                  // ②
+                  bool enabled = planeManager.enabled;
+                  foreach (var plane in planeManager.trackables)
+                  {
+                      plane.GetComponent<ARPlaneMeshVisualizer>().enabled = !enabled;
+                  }
+                  planeManager.enabled = !enabled;
                }
            }
        }

#endif
    }

    // ゴールを設置する関数
    void SetGoal(Vector3 position)
    {
        var rotation =
            Camera.main.transform.forward.normalized;
        var fixedPosition = position - rotation * 0.1f;
        goal.transform.position = fixedPosition;
    }
    // ボールを生成する関数
    void CreateBall(float flickDistance)
    {
        var worldPosition =
            Camera.main.ScreenToWorldPoint(
                Vector3.zero);
        var go = Instantiate(
            ball, worldPosition,
            Quaternion.identity);

        var rot =
            Camera.main.transform.forward.normalized;
        worldPosition += rot;
        rot.y += flickDistance / 200;
        var rb = go.GetComponent<Rigidbody>();
        rb.AddForce(
            rot * 2, ForceMode.VelocityChange);

        Destroy(go, 5.0f);
    }
}
```

①でARPlaneManagerの変数を追加し、Start関数の中で取得しています。こちらは平面検知を管理するスクリプトです。
②では、まずplaneManagerが有効であるかを変数として取得します。続いて、planeManager.trackablesには検出された平面のオブジェクトが格納されています。その中からARPlaneMeshVisualizer コンポーネントを取得しています。

　ARPlaneMeshVisualizerは 検出した平面を可視化するスクリプトです。こちらのenabledを切り替えことでARPlaneMeshVisualizerの内部で平面の表示非表示を切り替えてくれます。平面が非表示になっても平面のColliderはそのままなので、床にバウンドする処理はそのまま残ります。

`!enable`を代入することで、現在の値が有効であれば無効に、無効であれば有効に、交互に切り替えることができます。
上記処理を加えることで、平面の表示非表示を切り替えられるようになりました。