﻿using UnityEngine;
using System.Collections.Generic;
using UnityEngine.XR.ARFoundation;
using UnityEngine.XR.ARSubsystems;

public class GameManager : MonoBehaviour
{
    [SerializeField]
    GameObject goal;

    [SerializeField]
    GameObject ballPrefab;

    float maxRayDistance = 30.0f;

    ARRaycastManager raycastManager;
    List<ARRaycastHit> hitList
        = new List<ARRaycastHit>();

    private Vector3 touchStartPos;
    private Vector3 touchEndPos;

    void Start()
    {
        raycastManager
            = GetComponent<ARRaycastManager>();
    }
    void Update()
    {
#if UNITY_EDITOR
        // UnityEditor上でARの挙動をシミュレート
        if (Input.GetMouseButtonDown(0))
        {
            touchStartPos = Input.mousePosition;
        }
        if (Input.GetMouseButtonUp(0))
        {
            touchEndPos = Input.mousePosition;
            var flickDistance =
                touchEndPos.y - touchStartPos.y;
            if (flickDistance > 0)
            {
                CreateBall(flickDistance);
            }
            else
            {
                Ray ray = Camera.main.ScreenPointToRay(
                    Input.mousePosition);
                RaycastHit hit;
                if (Physics.Raycast(ray, out hit,
                    maxRayDistance))
                {
                    SetGoal(hit.point);
                }
            }
        }
#else
        // 端末上での動作
        if (Input.touchCount > 0)
        {
            var touch = Input.GetTouch(0);
            if (touch.phase == TouchPhase.Began)
            {
                touchStartPos = touch.position;
            }
            if (touch.phase == TouchPhase.Ended)
            {
                touchEndPos = touch.position;
                var flickDistance =
                    touchEndPos.y - touchStartPos.y;
                if (flickDistance > 0)
                {
                    CreateBall(flickDistance);
                }
                else
                {
                    if (raycastManager.Raycast(
                        touch.position, hitList,
                        TrackableType.Planes))
                    {
                        SetGoal(hitList[0].pose.position);
                    }
                }
            }
        }
#endif
    }

    // ゴールを設置するメソッド
    void SetGoal(Vector3 position)
    {
        var rotation =
            Camera.main.transform.forward.normalized;
        var fixedPosition = position - rotation * 0.1f;
        goal.transform.position = fixedPosition;
    }

    // ボールを生成するメソッド
    void CreateBall(float flickDistance)
    {
        var worldPosition =
            Camera.main.ScreenToWorldPoint(
                Vector3.zero);
        var go = Instantiate(
            ballPrefab, worldPosition,
            Quaternion.identity);

        var rotation =
            Camera.main.transform.forward.normalized;
        rotation.y += flickDistance / 200;
        var rb = go.GetComponent<Rigidbody>();
        rb.AddForce(
            rotation * 2, ForceMode.VelocityChange);

        Destroy(go, 5.0f);
    }
}