﻿using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;
using UnityEditor;

public class SnapScroll : ScrollRect
{
    [SerializeField]
    private float _smoothness = 10f;
    [SerializeField]
    private float _swipeWeight = 0.4f;
    [SerializeField]
    private float _flickWeight = 35f;
    [SerializeField]
    private int _cellNum = 3;
    [SerializeField]
    private float _cellWidth;

    private int _prevCellIndex = 0;
    private Vector2 _targetPosition;
    private bool _isSnap;
    private Vector2 _delta;

    private void Update()
    {
        if (_isSnap)
        {
            // Lerpで吸い付くアニメーション
            content.anchoredPosition = Vector2.Lerp(content.anchoredPosition, _targetPosition, Time.deltaTime * _smoothness);

            // 一定距離以下になったら終了
            if ((content.anchoredPosition - _targetPosition).magnitude < 0.01f)
            {
                content.anchoredPosition = _targetPosition;
                _isSnap = false;
            }
        }
    }

    public override void OnBeginDrag(PointerEventData eventData)
    {
        base.OnBeginDrag(eventData);
        _isSnap = false;
    }

    public override void OnDrag(PointerEventData eventData)
    {
        base.OnDrag(eventData);
        // フレーム間の移動距離を保存
        _delta = eventData.delta;
    }

    public override void OnEndDrag(PointerEventData eventData)
    {
        base.OnEndDrag(eventData);

        StopMovement();

        float weight = content.anchoredPosition.x / _cellWidth + _prevCellIndex;
        int cellIndex = _prevCellIndex;

        if (Mathf.Abs(weight) > _swipeWeight)
        {
            // スワイプ時のウェイト判定
            cellIndex += CalcCellIndexDiff(weight);
        }
        else if (Mathf.Abs(_delta.x) > _flickWeight)
        {
            // フリック時のウェイト判定
            cellIndex += CalcCellIndexDiff(_delta.x);
        }

        ScrollTo(cellIndex);

        _delta = Vector2.zero;
    }

    private int CalcCellIndexDiff(float weight)
    {
        if (weight < 0f)
        {
            return 1;
        }
        if (weight > 0f)
        {
            return -1;
        }
        return 0;
    }

    public void ScrollTo(int index)
    {
        index = Mathf.Clamp(index, 0, _cellNum - 1);

        // スナップするインデックスの位置を算出
        float nextPosX = index * _cellWidth * -1f;
        _targetPosition = new Vector2(nextPosX, content.anchoredPosition.y);

        _isSnap = true;

        _prevCellIndex = index;
    }
}

[CustomEditor(typeof(SnapScroll))]
public class SnapScrollEditor : Editor
{
    public override void OnInspectorGUI()
    {
        base.OnInspectorGUI();
    }
}