﻿// ※注意
// 使用するにはAssetStoreからDOTweenをダウンロードしインポートしてください

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using DG.Tweening;

public class UISample5
{
    [SerializeField]
    private Image _image;
    [SerializeField]
    private Text _text;

    public void PlayMoveAnimation()
    {
        //  座標(100, 100, 0)へ1秒で移動
        _image.transform.DOMove(new Vector3(100f, 100f, 0f), 1f)
            .Play();
    }

    public void PlayScaleAnimation()
    {
        // 2倍の大きさへ1秒で拡大
        _image.transform.DOScale(new Vector3(2f, 2f, 1f), 1f)
            .SetEase(Ease.OutQuart)
            .Play();
    }

    public void PlayColorAnimation()
    {
        // 初期色から赤、赤から初期色を無限ループする
        _image.DOColor(Color.red, 1f)
            .SetLoops(-1, LoopType.Yoyo)
            .Play();
    }

    /*
    // Killする処理を使うための実装
    private Tween _tween;
    public void PlayColorAnimation()
    {
        // 初期色から赤、赤から初期色を無限ループする
        _tween = _image.DOColor(Color.red, 1f)
            .SetLoops(-1, LoopType.Yoyo)
            .Play();
    }
    public void StopColorAnimation()
    {
        if (_tween != null) {
            _tween.Kill();
        }
    }
    */

    public void PlayRotateAnimation()
    {
        // 1秒で反時計回りに一周しない例
        //_image.transform.DORotate(new Vector3(0f, 0f, 360f), 1f)
        //    .Play();

        // 1秒で反時計回りに一周
        _image.transform.DORotate(new Vector3(0f, 0f, 360f), 1f, RotateMode.LocalAxisAdd)
            .Play();
    }

    /*
    // DOTween.Toを使用したPlayMoveAnimation
    public void PlayMoveAnimation()
    {
        // 座標(-200, 0, 0)から座標(100, 100, 0)へ1秒で移動
        DOTween.To(() => new Vector3(-200f, 0f, 0f), pos => _image.transform.localPosition = pos, new Vector3(100f, 100f, 0f), 1f)
            .Play();
    }
    */

    public void PlayTextAnimation()
    {
        // 0から10まで1秒でテキストをカウントアップさせる
        DOTween.To(() => 0, num => _text.text = num.ToString(), 10, 1f)
            .Play();
    }

    public void PlayAnimation()
    {
        Sequence seq = DOTween.Sequence();
        // Append:現在のSequenceに追加
        seq.Append(_image.DOFade(0f, 0.5f));
        // Join:現在のSequenceの最後尾と同じ位置に追加
        seq.Join(_image.transform.DOLocalMoveX(0f, 0.5f));
        // Insert:指定位置に追加
        seq.Insert(0.5f, _image.DOFade(1f, 0.5f));
        // コールバック:Append,Join,Insert可能
        seq.AppendCallback(() =>
        {
            _image.color = Color.red;
        });
        // インターバル:Append,Insert可能
        seq.AppendInterval(0.5f);
        // Tweenと同じ設定メソッドを使用できる
        seq.OnComplete(() =>
        {
            _image.enabled = false;
        });
        seq.Play();
    }

    /*
    // Killする処理を使うための実装
    private Sequence _seq;
    public void PlayAnimation()
    {
        //  すでにSequenceが動いていたら
        // Killしてから新しいSequenceを作る
        _seq.SafeKill();
        _seq = DOTween.Sequence();
        _seq.Append(...);
        .
        .
        .
    }
    */
}